@@ -17,16 +17,7 @@ class TunnelRouter(object):
             return self._conf.hosts[address[0]], address[1]
         return address
 
-    async def select(self, address):
-        address_list = [address]
-        if not utils.is_ip_address(address[0]):
-            resolved_address = await self.resolve(address)
-            if resolved_address != address:
-                address_list.append(resolved_address)
-            resolved_address = await utils.resolve_address(address)
-            if resolved_address != address:
-                address_list.append(resolved_address)
-
+    async def select_tunnel(self, address_list):
         for rule in self._conf.rules:
             for address in address_list:
                 if await rule.is_hit(address):
@@ -36,6 +27,24 @@ class TunnelRouter(object):
                         return "block", None
                     else:
                         return rule.id, tunnel
-        else:
-            # select default tunnel
-            return "default", self._conf.default_tunnel
+        return None, None
+
+    async def select(self, address):
+        address_list = [address]
+        is_ip_address = utils.is_ip_address(address[0])
+        if not is_ip_address:
+            resolved_address = await self.resolve(address)
+            if resolved_address != address:
+                address_list.append(resolved_address)
+
+        rule, tunnel = await self.select_tunnel(address_list)
+        if rule:
+            return rule, tunnel
+        if not is_ip_address:
+            resolved_address = await utils.resolve_address(address)
+            if resolved_address != address:
+                rule, tunnel = await self.select_tunnel(address_list)
+                if rule:
+                    return rule, tunnel
+        # select default tunnel
+        return "default", self._conf.default_tunnel
